package java.awt.dnd;

import java.io.IOException;
import java.awt.datatransfer.UnsupportedFlavorException;
import sun.awt.datatransfer.TransferableProxy;
import java.util.Arrays;
import java.util.List;
import java.awt.datatransfer.DataFlavor;
import java.awt.Component;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.peer.DropTargetContextPeer;
import java.io.Serializable;

public class DropTargetContext implements Serializable
{
    private static final long serialVersionUID = -634158968993743371L;
    private DropTarget dropTarget;
    private transient DropTargetContextPeer dropTargetContextPeer;
    private transient Transferable transferable;
    
    DropTargetContext(DropTarget dropTarget) {
        this.dropTarget = dropTarget;
    }
    
    public DropTarget getDropTarget() {
        return this.dropTarget;
    }
    
    public Component getComponent() {
        return this.dropTarget.getComponent();
    }
    
    public void addNotify(DropTargetContextPeer dropTargetContextPeer) {
        this.dropTargetContextPeer = dropTargetContextPeer;
    }
    
    public void removeNotify() {
        this.dropTargetContextPeer = null;
        this.transferable = null;
    }
    
    protected void setTargetActions(int targetActions) {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        if (dropTargetContextPeer != null) {
            synchronized (dropTargetContextPeer) {
                dropTargetContextPeer.setTargetActions(targetActions);
                this.getDropTarget().doSetDefaultActions(targetActions);
            }
        }
        else {
            this.getDropTarget().doSetDefaultActions(targetActions);
        }
    }
    
    protected int getTargetActions() {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        return (dropTargetContextPeer != null) ? dropTargetContextPeer.getTargetActions() : this.dropTarget.getDefaultActions();
    }
    
    public void dropComplete(boolean b) throws InvalidDnDOperationException {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        if (dropTargetContextPeer != null) {
            dropTargetContextPeer.dropComplete(b);
        }
    }
    
    protected void acceptDrag(int n) {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        if (dropTargetContextPeer != null) {
            dropTargetContextPeer.acceptDrag(n);
        }
    }
    
    protected void rejectDrag() {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        if (dropTargetContextPeer != null) {
            dropTargetContextPeer.rejectDrag();
        }
    }
    
    protected void acceptDrop(int n) {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        if (dropTargetContextPeer != null) {
            dropTargetContextPeer.acceptDrop(n);
        }
    }
    
    protected void rejectDrop() {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        if (dropTargetContextPeer != null) {
            dropTargetContextPeer.rejectDrop();
        }
    }
    
    protected DataFlavor[] getCurrentDataFlavors() {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        return (dropTargetContextPeer != null) ? dropTargetContextPeer.getTransferDataFlavors() : new DataFlavor[0];
    }
    
    protected List<DataFlavor> getCurrentDataFlavorsAsList() {
        return Arrays.asList(this.getCurrentDataFlavors());
    }
    
    protected boolean isDataFlavorSupported(DataFlavor dataFlavor) {
        return this.getCurrentDataFlavorsAsList().contains(dataFlavor);
    }
    
    protected Transferable getTransferable() throws InvalidDnDOperationException {
        DropTargetContextPeer dropTargetContextPeer = this.getDropTargetContextPeer();
        if (dropTargetContextPeer == null) {
            throw new InvalidDnDOperationException();
        }
        if (this.transferable == null) {
            Transferable transferable = dropTargetContextPeer.getTransferable();
            int transferableJVMLocal = dropTargetContextPeer.isTransferableJVMLocal();
            synchronized (this) {
                if (this.transferable == null) {
                    this.transferable = this.createTransferableProxy(transferable, (boolean)transferableJVMLocal);
                }
            }
        }
        return this.transferable;
    }
    
    DropTargetContextPeer getDropTargetContextPeer() {
        return this.dropTargetContextPeer;
    }
    
    protected Transferable createTransferableProxy(Transferable transferable, boolean b) {
        return new DropTargetContext.TransferableProxy(transferable, b);
    }
    
    protected class TransferableProxy implements Transferable
    {
        protected Transferable transferable;
        protected boolean isLocal;
        private sun.awt.datatransfer.TransferableProxy proxy;
        
        TransferableProxy(Transferable transferable, boolean b) {
            this.proxy = new sun.awt.datatransfer.TransferableProxy(transferable, b);
            this.transferable = transferable;
            this.isLocal = b;
        }
        
        @Override
        public DataFlavor[] getTransferDataFlavors() {
            return this.proxy.getTransferDataFlavors();
        }
        
        @Override
        public boolean isDataFlavorSupported(DataFlavor flavor) {
            return this.proxy.isDataFlavorSupported(flavor);
        }
        
        @Override
        public Object getTransferData(DataFlavor df) throws UnsupportedFlavorException, IOException {
            return this.proxy.getTransferData(df);
        }
    }
}
